local AddonName, SSTB = ...

local barCount = 5


-- Bar numbers are order in which Blizzard creates the bars
local Reputation = 1
local Honor = 2
local Artifact = 3
local Experience = 4
local Azerite = 5

local CycleTime = 7
local extendedTime = 20

local TrackedBar = UnitLevel("player") < 120 and Experience or Azerite
local manager,bar
local ReputationTable = {}

local frame = CreateFrame("Frame", nil, UIParent)
--frame:RegisterEvent("PLAYER_ENTERING_WORLD")


local e = 0
local extended = false -- Set when changed by event for longer tracking of bar

-- Addon running section to automatically cycle bars on timer
local OnUpdate = function(self, elapsed)
	e = e + elapsed
	if (not(extended) and e < CycleTime) or (extended and e < extendedTime) then
		return
	end
	extended = false
	
	TrackedBar = TrackedBar +1
	if TrackedBar > barCount then
		TrackedBar = 1
	end

	-- Original conditions of ShouldBeVisible per bar
	-- After time has expired it will attempt to find a new bar
	-- If it fails it will return and try next frame for next bar
	-- and so on...
	if TrackedBar == Reputation then
		local name, reaction, minFaction, maxFaction, value, factionID = GetWatchedFactionInfo();
		if not(name) then 
			return
		end
	elseif TrackedBar == Honor then
		if not(IsWatchingHonorAsXP() or C_PvP.IsActiveBattlefield() or IsInActiveWorldPVP()) then
			return
		end
	elseif TrackedBar == Artifact then
		if HasArtifactEquipped() and not C_ArtifactUI.IsEquippedArtifactMaxed() and not C_ArtifactUI.IsEquippedArtifactDisabled() then
		-- let the bar show
		else
			return
		end
	elseif TrackedBar == Experience then
		if IsPlayerAtEffectiveMaxLevel() or IsXPUserDisabled() then
			return
		end
	elseif TrackedBar == Azerite then
		local IsMaxLevel = C_AzeriteItem.IsAzeriteItemAtMaxLevel();
		if C_AzeriteItem.HasActiveAzeriteItem() and not IsMaxLevel then
		-- let the bar show
		else
			return
		end
	else -- shouldn't get here but just in case
		return
	end
	
	e = 0 -- we made it through bar selection with an active bar so reset timer now
	
	-- update bar shown to new bar
	manager:UpdateBarsShown()
	bar[TrackedBar]:OnEnter()
	bar[TrackedBar]:OnLeave()
	bar[TrackedBar]:ShowText()
	bar[TrackedBar]:Update()
	if bar[TrackedBar].UpdateCurrentText then
		bar[TrackedBar]:UpdateCurrentText()
	end
	
	
end

-- Addon Initialization setup
local Initialize = function(self, elapsed)
	if e < 30 then 
		e = e + 1
	-- wait 30 frames for the bars to be loaded and displaying properly before hooking and changing
	elseif e == 30 then
		e = 0 -- e still used after this so reset it
		AZERITE_POWER_BAR = "Azerite Power %s" -- Why was it set to "Artifact Power %s"
		manager = _G["StatusTrackingBarManager"] -- Blizzard's default bar manager
		if _G["BT4StatusBarTrackingManager"] then -- Bartender 4's bar manager
			manager = _G["BT4StatusBarTrackingManager"]
		end
		
		bar = manager.bars
	
		-- Change bar visibility to be controlled by this addon
		bar[1].ShouldBeVisible = function(self)
			return TrackedBar == Reputation
		end
			
		bar[2].ShouldBeVisible = function(self)
			return TrackedBar == Honor
		end
				
		bar[3].ShouldBeVisible = function(self)
			return TrackedBar == Artifact
		end
				
		bar[4].ShouldBeVisible = function(self)
			return TrackedBar == Experience
		end
				
		bar[5].ShouldBeVisible = function(self)
			return TrackedBar == Azerite
		end
				
		manager:SetInitialBarSize();
		manager:UpdateBarsShown();

		-- Grab the events that are required to know when to change bars
		frame:RegisterEvent("UPDATE_FACTION")
		frame:RegisterEvent("HONOR_XP_UPDATE")
		frame:RegisterEvent("CHAT_MSG_COMBAT_HONOR_GAIN")
		frame:RegisterEvent("ARTIFACT_XP_UPDATE")
		frame:RegisterEvent("CHAT_MSG_COMBAT_XP_GAIN")
		frame:RegisterEvent("PLAYER_LEVEL_UP")
		frame:RegisterEvent("PLAYER_XP_UPDATE")
		frame:RegisterEvent("AZERITE_ITEM_EXPERIENCE_CHANGED")

		-- Pre-scan factions known for current rep as Blizzard event doesn't tell which faction had rep change during event
		local numFactions = GetNumFactions()
		for factionIndex = 1, numFactions do
			local name, description, standingId, bottomValue, topValue, earnedValue, atWarWith, canToggleAtWar, isHeader, isCollapsed, hasRep, isWatched, isChild, factionID, hasBonusRepGain, canBeLFGBonus = GetFactionInfo(factionIndex)
			if isHeader and isCollapsed then
				ExpandFactionHeader(factionIndex)
				numFactions = GetNumFactions()
			end
			if hasRep or not isHeader then
				-- Store rep values to compare with later
				ReputationTable[factionID] = earnedValue
			end
		end
		
		frame:SetScript("OnUpdate", OnUpdate) -- Set OnUpdate script to running script instead of loading script
		Loading = nil -- set for garbage collection of Loading script
		return
	end
end
frame:SetScript("OnUpdate", Initialize)

-- Addon section to change bars on change of bar value
frame:SetScript("OnEvent",function(self, event, ...)
	-- local args = {...} not using any args at the moment...
	
	-- Reputation
	if event == "UPDATE_FACTION" then
		TrackedBar = Reputation
		
		-- Scan factions for current rep
		local numFactions = GetNumFactions()
		for factionIndex = 1, numFactions do
			local name, description, standingId, bottomValue, topValue, earnedValue, atWarWith, canToggleAtWar, isHeader, isCollapsed, hasRep, isWatched, isChild, factionID, hasBonusRepGain, canBeLFGBonus = GetFactionInfo(factionIndex)
			if isHeader and isCollapsed then
				ExpandFactionHeader(factionIndex)
				numFactions = GetNumFactions()
			end
			if hasRep or not isHeader then
				if ReputationTable[factionID] ~= earnedValue then -- Found a changed rep so set it to tracked
					ReputationTable[factionID] = earnedValue
					SetWatchedFactionIndex(factionIndex)
				end
			end
		end
	
	-- Honor
	elseif event == "HONOR_XP_UPDATE" or
	       event == "CHAT_MSG_COMBAT_HONOR_GAIN" then
		TrackedBar = Honor
	
	-- Artifact
	elseif event == "ARTIFACT_XP_UPDATE" then
		TrackedBar = Artifact
	
	-- Experience
	elseif event == "CHAT_MSG_COMBAT_XP_GAIN" or
           event == "PLAYER_LEVEL_UP" or
		   event == "PLAYER_XP_UPDATE" then
		TrackedBar = Experience
		   
	-- Azerite
	elseif event == "AZERITE_ITEM_EXPERIENCE_CHANGED" then
		TrackedBar = Azerite
		
	else-- Why are we here?
		return
	end
	
	e = 0
	extended = true
	manager:UpdateBarsShown()
	bar[TrackedBar]:OnEnter()
	bar[TrackedBar]:OnLeave()
	--manager:UpdateTextVisibility()
	bar[TrackedBar]:ShowText()
	bar[TrackedBar]:Update()
	if bar[TrackedBar].UpdateCurrentText then
		bar[TrackedBar]:UpdateCurrentText()
	end
end)



	



